"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pitch = exports.raw = void 0;
const schema_utils_1 = require("schema-utils");
const sharp_1 = __importDefault(require("sharp"));
const webpack_loader_util_1 = require("@calvin-l/webpack-loader-util");
const getMaxDensity_1 = __importDefault(require("./helpers/getMaxDensity"));
const getOptionFromSize_1 = __importDefault(require("./helpers/getOptionFromSize"));
const normalizeQueryLoaderOptions_1 = __importDefault(require("./helpers/normalizeQueryLoaderOptions"));
const validateSizes_1 = __importDefault(require("./helpers/validateSizes"));
const options_json_1 = __importDefault(require("./options.json"));
exports.raw = true;
function pitch(remainingRequest) {
    const callback = this.async();
    const defaultOptions = {
        // @ts-expect-error setting sizes as undefined and let validate throw error if it doesn't exist
        sizes: undefined,
        scaleUp: false,
        esModule: true,
    };
    const options = {
        ...defaultOptions,
        ...webpack_loader_util_1.getOptions(this),
    };
    schema_utils_1.validate(options_json_1.default, options, {
        name: "webpack-image-srcset-loader",
        baseDataPath: "options",
    });
    validateSizes_1.default(options.sizes);
    generateSrcSetString(remainingRequest, this.loaders, this.loaderIndex, options, this.fs, this.resourcePath)
        .then((srcSetString) => {
        callback(null, `${options.esModule ? "export default" : "module.exports ="}  \`${srcSetString}\`;`);
    })
        .catch((e) => {
        throw e;
    });
}
exports.pitch = pitch;
function default_1(source) {
    return source;
}
exports.default = default_1;
async function generateSrcSetString(remainingRequest, loaders, loaderIndex, options, fs, resourcePath) {
    let result = "";
    let width;
    const sizes = options.sizes;
    const maxDensity = getMaxDensity_1.default(sizes);
    const separator = ", ";
    const requireStart = "${require('-!";
    const requireEnd = "')}";
    for (const size of sizes) {
        if (size === "original") {
            result += `${requireStart}${addOptionsToResizeLoader(remainingRequest, loaders, loaderIndex, {}, options.customOptionsFactory)}${requireEnd}${separator}`;
            continue;
        }
        const resizeLoaderOption = getOptionFromSize_1.default(size, maxDensity);
        if (!options.scaleUp && resizeLoaderOption.width !== undefined) {
            if (width === undefined) {
                const buffer = fs.readFileSync(resourcePath);
                width = (await sharp_1.default(buffer).metadata()).width;
            }
            if (width !== undefined && resizeLoaderOption.width > width)
                continue;
        }
        result += `${requireStart}${addOptionsToResizeLoader(remainingRequest, loaders, loaderIndex, resizeLoaderOption, options.customOptionsFactory)}${requireEnd} ${size}${separator}`;
    }
    result = result.substring(0, result.length - 2);
    return result;
}
function addOptionsToResizeLoader(remainingRequest, loaders, loaderIndex, options, customOptionsFactory) {
    var _a;
    const nextLoader = loaders[loaderIndex + 1];
    const isNextLoaderQueryLoader = getIsLoaderQueryLoader(nextLoader);
    const resizeLoaderOptions = nextLoader.options;
    const resizeLoaderRequest = nextLoader.request;
    const resizeLoaderPath = nextLoader.path;
    if (customOptionsFactory)
        return remainingRequest.replace(resizeLoaderRequest, resizeLoaderPath +
            "?" +
            escapeJsonStringForLoader(JSON.stringify(customOptionsFactory(options.width, options.scale, resizeLoaderOptions))));
    if (isNextLoaderQueryLoader) {
        const queryLoaderOptions = normalizeQueryLoaderOptions_1.default(resizeLoaderOptions);
        return remainingRequest.replace(resizeLoaderRequest, resizeLoaderPath +
            "?" +
            escapeJsonStringForLoader(JSON.stringify({
                ...queryLoaderOptions,
                use: {
                    loader: queryLoaderOptions.use.loader,
                    options: {
                        scaleUp: true,
                        ...queryLoaderOptions.use.options,
                        ...options,
                        fileLoaderOptions: {
                            ...(_a = queryLoaderOptions.use.options) === null || _a === void 0 ? void 0 : _a.fileLoaderOptions,
                            esModule: false,
                        },
                    },
                },
            })));
    }
    return remainingRequest.replace(resizeLoaderRequest, resizeLoaderPath +
        "?" +
        escapeJsonStringForLoader(JSON.stringify({
            scaleUp: true,
            ...resizeLoaderOptions,
            ...options,
            fileLoaderOptions: {
                ...resizeLoaderOptions === null || resizeLoaderOptions === void 0 ? void 0 : resizeLoaderOptions.fileLoaderOptions,
                esModule: false,
            },
        })));
}
function getIsLoaderQueryLoader(nextLoader) {
    if (nextLoader === undefined)
        throw "webpack-image-srcset-loader must be placed at the beginning and followed by a resize loader";
    if (nextLoader.path.includes("webpack-query-loader"))
        return true;
    return false;
}
// needed so webpack doesn't mistake "!" for query operator "!"
function escapeJsonStringForLoader(s) {
    return s.replace(/!/g, "\\\\x21");
}
//# sourceMappingURL=index.js.map